﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Web.Razor;
using System.Web.Razor.Generator;
using System.IO;
using Microsoft.CSharp;
using System.CodeDom.Compiler;
using System.Reflection;
namespace RazorDemo
{
    public class Engine
    {
        public string LastGeneratedCode { get; set; }
        public string Execute(string TemplateCode, string defaultnamespace, string defaultclassname,dynamic Model)
        {
            string resultstring = null;
            var parseresult = ParseToCode(TemplateCode, defaultnamespace, defaultclassname);
            resultstring = ExecuteInternal(parseresult, defaultnamespace, defaultclassname,Model);

            return resultstring;
        }
        private string ExecuteInternal(GeneratorResults razorResults, string defaultnamespace, string defaultclassname, dynamic Model)
        {
            
            using (var provider = new CSharpCodeProvider())
            {
                var compiler = new CompilerParameters(); 
                compiler.ReferencedAssemblies.Add("System.dll");
                compiler.ReferencedAssemblies.Add("System.Core.dll");
                compiler.ReferencedAssemblies.Add("Microsoft.CSharp.dll");
                compiler.ReferencedAssemblies.Add(Assembly.GetExecutingAssembly().Location);
                compiler.GenerateInMemory = true;
                var result = provider.CompileAssemblyFromDom(compiler, razorResults.GeneratedCode);
                if (result.Errors.HasErrors) 
                {
                    var error = result.Errors.OfType<CompilerError>().Where(i => !i.IsWarning).FirstOrDefault();
                    if (error != null) throw new Exception(error.ErrorText); //抛出错误
                }
                TemplateBase temp= (TemplateBase)result.CompiledAssembly.CreateInstance(defaultnamespace+"."+defaultclassname);
                temp.Model = Model;
                try
                {
                    temp.Execute();
                }
                catch (Exception ex)
                {
                    throw new Exception("执行错误",ex);
                }
                return temp.Output.ToString();
            }
            

        }
        public GeneratorResults ParseToCode(string TemplateCode,string defaultnamespace,string defaultclassname)
        {
            GeneratorResults razorResults;
            var host = new RazorEngineHost(new CSharpRazorCodeLanguage());
            host.DefaultBaseClass = typeof(TemplateBase).FullName;
            host.DefaultNamespace =defaultnamespace;
            host.DefaultClassName = defaultclassname;
            host.NamespaceImports.Add("System");
            host.GeneratedClassContext = new GeneratedClassContext("Execute","Write", "WriteLiteral");
        
            
            var engine = new RazorTemplateEngine(host);
            using (var reader = new StringReader(TemplateCode))
            {
                razorResults = engine.GenerateCode(reader);
                
                CSharpCodeProvider codeProvider = new CSharpCodeProvider();
                CodeGeneratorOptions options = new CodeGeneratorOptions();
                options.BracingStyle = "C";
               
               
                using (StringWriter writer = new StringWriter())
                {
                    IndentedTextWriter indentwriter = new IndentedTextWriter(writer, "    ");
                   
                    codeProvider.GenerateCodeFromCompileUnit(razorResults.GeneratedCode, indentwriter, options);
                    indentwriter.Flush();
                    indentwriter.Close();
                    LastGeneratedCode = writer.GetStringBuilder().ToString();
                    

                }
                
            }
            return razorResults;
           
        }
       
    }
}
